% Demo program for NRSfM_PND.
%
% Ref: Minsik Lee, Jungchan Cho, Chong-Ho Choi, and Songhwai Oh,
% "Procrustean Normal Distribution for Non-Rigid Structure from Motion,"
% CVPR 2013, Portland, Oregon, June 23-28, 2013.
%
% Authors: Minsik Lee (mlee.paper@gmail.com), Jungchan Cho (jungchan.cho@gmail.com)
% Last update: 2013-08-08
% License: GPLv3

%
% Copyright (C) 2013 Minsik Lee, Jungchan Cho
% This file is part of NRSfM_PND.
%
% NRSfM_PND is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% NRSfM_PND is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with NRSfM_PND.  If not, see <http://www.gnu.org/licenses/>.



% **IMPORTANT: Run 'download_and_rearrange_data.m' at the first time to download data sets.
% If you are going to use another data set for this demo, the data should be rearranged as
%
% X(:, :, k) = [x_k1 x_k2 ... x_kp; y_k1 y_k2 ... y_kp; z_k1 z_k2 ... z_kp]
%
% Here, x_ki and y_ki are the observed 2D coordinates of the kth frame, and z_ki are the
% corresponding depth coordinates. Then, the observation data D will be generated as
% D(:, :, k) = [X(1:2, :, k); zeros(1, p)]. W is the weight mask that indicates whether the
% elements are missing or not. (false = missing)


clear; close all; clc;


% Experimental setting
seq = 'walking';    % choice of data set
noise = 0;          % noise level
rmiss = 0;          % missing rate


% Loading
load(['Data/' seq '_rearranged']);
[k p nSample] = size(X);

% Input data generation
D = zeros(k, p, nSample);
temp = X(1:2, :, :);
D(1:2, :, :) = temp + noise*max(abs(reshape(temp-repmat(mean(temp, 2), 1, p), [], 1)))*randn(2, p, nSample);

W = true(k, p, nSample);
W(3, :, :) = false;

ind = rand(p*nSample, 1) < rmiss;
D(1:2, ind) = 0;
W(1:2, ind) = false;

tic;

% NRSfM: EM-PND (Procrustean Normal Distribution)
[rX s R M C var_D] = NRSfM_PND(D, W);

toc;

% Evaluation
gX = X - repmat(mean(X, 2), 1, p);
perf = zeros(1, nSample);
for i=1:nSample
    perf(i) = sqrt(min(mse(gX(:, :, i)-rX(:, :, i)), mse(gX(:, :, i)-[rX(1:2, :, i); -rX(3, :, i)]))/mse(gX(:, :, i)));
end
disp(['mean error : ' num2str(mean(perf))]);

% Plot results
plot_NRSfM(D, W, gX, rX);

